#!/bin/bash
###########################################################################
#                                                                         #
#                                                                         #
#              Copyright (c) 2011 Iskratel                                #
#                                                                         #
#                                                                         #
#   Name        : itConfigCheck.sh                                        #
#                                                                         #
#   Description : Script for checking (and protecting with) a config      #
#                 script file checksum.                                   #
#                                                                         #
#   Date        : May 2011                                                #
#                                                                         #
#   Author      : Andrej Furlan, Metalab                                  #
#                                                                         #
#                                                                         #
###########################################################################


CHECKSUM_PREFIX="! checksum="
CHECKSUM_REGEX="^"${CHECKSUM_PREFIX}"(.*)$"

SILENT=0     # silent mode? (no printouts)
CALC=0       # calculate checksum and append it to file?
FILENAME=""  # the config file to work on

EXIT_CHECKSUM_NOK=3

USAGE="Usage: $0 [--silent] [--protect] <config-file>"
USAGE=$USAGE"\n"
USAGE=$USAGE"\n  Checks if checksum appended to the end of the <config-file> is OK"
USAGE=$USAGE"\n  The checksum is present at the last line as a script comment."
USAGE=$USAGE"\n  If no checksum present, then verification succeeds."
USAGE=$USAGE"\n  If --protect option is present, then checksum is not checked but it"
USAGE=$USAGE"\n  is calculated and appended to the end of the script."
USAGE=$USAGE"\n"
USAGE=$USAGE"\n  --silent - silent mode (no printout)"
USAGE=$USAGE"\n  --protect - protect the file with checksum (deleting the old one if present)."
USAGE=$USAGE"\n  <config-file> - path to config file"
USAGE=$USAGE"\n"
USAGE=$USAGE"\n  Return values: 0-success, 1-error, 2-invalid parameters, $EXIT_CHECKSUM_NOK-checksum NOT OK"
USAGE=$USAGE"\n"



# printErr(errStr)
#   print error string errStr (if silent mode is not turned on)
function printErr()
{
    if [[ $SILENT -eq 0 ]]; then
        echo -e $1
    fi
}

function exitOnErr()
{
    if [[ $# -ne 1 ]]; then
        echo "FUBAR: exitOnErr(): invalid arguments"
        exit 1
    fi
    
    EXIT_CODE=$1
    if [[ $EXIT_CODE -ne 0 ]]; then
        printErr "Error occured. Exiting."
        exit 1
    fi
    return 0
}

# checksum(filename, numLines):
#   Calculate checksum of numLines of file filename.
#   The calculated checksum is set to variable CHECKSUM on success.
function checksum()
{
    if [[ $# -ne 2 ]]; then
        echo "FUBAR: checksum(): invalid arguments"
        exit 1
    fi

    local FILENAME=$1
    local NUM_LINES=$2

    CHECKSUM=`head -n $NUM_LINES $FILENAME | md5sum | sed -r s/' -.*$'//`
    return $?
}



# Parse command line arguments:
CMDLINE_ERR=0
for arg in "$@"
do
    if [[ ${arg:0:1} == "-" ]]; then
        if [[ $arg == "--protect" ]]; then
            CALC=1
        elif [[ $arg == "--silent" ]]; then
            SILENT=1    
        else
            CMDLINE_ERR=1
        fi
    else
        if [[ -z $FILENAME ]]; then
            FILENAME=$arg
        else
            CMDLINE_ERR=1
        fi
    fi
done
if [[ -z $FILENAME ]]; then
    CMDLINE_ERR=1
fi
if [[ $CMDLINE_ERR -eq 1 ]]; then
    printErr "$USAGE"
    exit 2
fi


# Count num lines:
NUM_LINES=`cat $FILENAME | wc -l`


CHECKSUM_PRESENT=`tail -n 1 $FILENAME | grep -Ec "${CHECKSUM_REGEX}"`
if [[ $CALC -eq 1 ]]; then  # 'Calculate checksum' mode:        
    # If checksum is already present, delete it first.
    if [[ $CHECKSUM_PRESENT -eq 1 ]]; then
        sed -i "\$ d" $FILENAME  # Delete the last line (where checksum is)
    fi
    
    # Calculate checksum and append it to the end of the file.
    checksum $FILENAME $NUM_LINES
    sed -i "\$ a\\${CHECKSUM_PREFIX}${CHECKSUM}" $FILENAME
    exit 0
else # 'Verify checksum' mode
    if [[ $CHECKSUM_PRESENT -eq 0 ]]; then
        # Trivial: no checksum, nothing to check.
        exit 0
    fi
    
    # Extract current saved checksum:
    SAVED_CHECKSUM=`tail -n 1 $FILENAME | sed -r s/"${CHECKSUM_REGEX}"/'\1'/`
    # Real checksum:
    checksum $FILENAME $(( $NUM_LINES - 1 ))
    if [[ $SAVED_CHECKSUM == $CHECKSUM ]]; then
        exit 0
    else
        printErr "$FILENAME: Checksum NOT OK"
        exit $EXIT_CHECKSUM_NOK
    fi
fi

exit 0


